@@ -1,7 +1,7 @@
 /* $OpenBSD$ */
 
 /*
- * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
+ * Copyright (c) 2007 Nicholas Marriott <nicholas.marriott@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -40,24 +40,21 @@
  * Main server functions.
  */
 
-struct clients	 clients;
+struct clients		 clients;
 
-int		 server_fd;
-int		 server_shutdown;
-struct event	 server_ev_accept;
+struct tmuxproc		*server_proc;
+int			 server_fd;
+int			 server_exit;
+struct event		 server_ev_accept;
 
-struct session		*marked_session;
-struct winlink		*marked_winlink;
-struct window		*marked_window;
-struct window_pane	*marked_window_pane;
-struct layout_cell	*marked_layout_cell;
+struct cmd_find_state	 marked_pane;
 
 int	server_create_socket(void);
-void	server_loop(void);
-int	server_should_shutdown(void);
-void	server_send_shutdown(void);
-void	server_accept_callback(int, short, void *);
-void	server_signal_callback(int, short, void *);
+int	server_loop(void);
+int	server_should_exit(void);
+void	server_send_exit(void);
+void	server_accept(int, short, void *);
+void	server_signal(int);
 void	server_child_signal(void);
 void	server_child_exited(pid_t, int);
 void	server_child_stopped(pid_t, int);
@@ -66,22 +63,18 @@ void	server_child_stopped(pid_t, int);
 void
 server_set_marked(struct session *s, struct winlink *wl, struct window_pane *wp)
 {
-	marked_session = s;
-	marked_winlink = wl;
-	marked_window = wl->window;
-	marked_window_pane = wp;
-	marked_layout_cell = wp->layout_cell;
+	cmd_find_clear_state(&marked_pane, NULL, 0);
+	marked_pane.s = s;
+	marked_pane.wl = wl;
+	marked_pane.w = wl->window;
+	marked_pane.wp = wp;
 }
 
 /* Clear marked pane. */
 void
 server_clear_marked(void)
 {
-	marked_session = NULL;
-	marked_winlink = NULL;
-	marked_window = NULL;
-	marked_window_pane = NULL;
-	marked_layout_cell = NULL;
+	cmd_find_clear_state(&marked_pane, NULL, 0);
 }
 
 /* Is this the marked pane? */
@@ -90,9 +83,9 @@ server_is_marked(struct session *s, struct winlink *wl, struct window_pane *wp)
 {
 	if (s == NULL || wl == NULL || wp == NULL)
 		return (0);
-	if (marked_session != s || marked_winlink != wl)
+	if (marked_pane.s != s || marked_pane.wl != wl)
 		return (0);
-	if (marked_window_pane != wp)
+	if (marked_pane.wp != wp)
 		return (0);
 	return (server_check_marked());
 }
@@ -101,25 +94,7 @@ server_is_marked(struct session *s, struct winlink *wl, struct window_pane *wp)
 int
 server_check_marked(void)
 {
-	struct winlink	*wl;
-
-	if (marked_window_pane == NULL)
-		return (0);
-	if (marked_layout_cell != marked_window_pane->layout_cell)
-		return (0);
-
-	if (!session_alive(marked_session))
-		return (0);
-	RB_FOREACH(wl, winlinks, &marked_session->windows) {
-		if (wl->window == marked_window && wl == marked_winlink)
-			break;
-	}
-	if (wl == NULL)
-		return (0);
-
-	if (!window_has_pane(marked_window, marked_window_pane))
-		return (0);
-	return (window_pane_visible(marked_window_pane));
+	return (cmd_find_valid_state(&marked_pane));
 }
 
 /* Create server socket. */
@@ -148,7 +123,7 @@ server_create_socket(void)
 		return (-1);
 	umask(mask);
 
-	if (listen(fd, 16) == -1)
+	if (listen(fd, 128) == -1)
 		return (-1);
 	setblocking(fd, 0);
 
@@ -161,36 +136,24 @@ server_start(struct event_base *base, int lockfd, char *lockfile)
 {
 	int	pair[2];
 
-	/* The first client is special and gets a socketpair; create it. */
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pair) != 0)
 		fatal("socketpair failed");
-	log_debug("starting server");
 
-	switch (fork()) {
-	case -1:
-		fatal("fork failed");
-	case 0:
-		break;
-	default:
+	server_proc = proc_start("server", base, 1, server_signal);
+	if (server_proc == NULL) {
 		close(pair[1]);
 		return (pair[0]);
 	}
 	close(pair[0]);
 
-	/*
-	 * Must daemonise before loading configuration as the PID changes so
-	 * $TMUX would be wrong for sessions created in the config file.
-	 */
-	if (daemon(1, 0) != 0)
-		fatal("daemon failed");
+	if (log_get_level() > 3)
+		tty_create_log();
 
-	/* event_init() was called in our parent, need to reinit. */
-	clear_signals(0);
-	if (event_reinit(base) != 0)
-		fatal("event_reinit failed");
-
-	logfile("server");
-	log_debug("server started, pid %ld", (long) getpid());
+#ifdef __OpenBSD__
+	if (pledge("stdio rpath wpath cpath fattr unix getpw recvfd proc exec "
+	    "tty ps", NULL) != 0)
+		fatal("pledge failed");
+#endif
 
 	RB_INIT(&windows);
 	RB_INIT(&all_window_panes);
@@ -199,13 +162,8 @@ server_start(struct event_base *base, int lockfd, char *lockfile)
 	TAILQ_INIT(&session_groups);
 	mode_key_init_trees();
 	key_bindings_init();
-	utf8_build();
 
-	start_time = time(NULL);
-	log_debug("socket path %s", socket_path);
-#ifdef HAVE_SETPROCTITLE
-	setproctitle("server (%s)", socket_path);
-#endif
+	gettimeofday(&start_time, NULL);
 
 	server_fd = server_create_socket();
 	if (server_fd == -1)
@@ -213,9 +171,11 @@ server_start(struct event_base *base, int lockfd, char *lockfile)
 	server_update_socket();
 	server_client_create(pair[1]);
 
-	unlink(lockfile);
-	free(lockfile);
-	close(lockfd);
+	if (lockfd >= 0) {
+		unlink(lockfile);
+		free(lockfile);
+		close(lockfd);
+	}
 
 	start_cfg();
 
@@ -223,32 +183,20 @@ server_start(struct event_base *base, int lockfd, char *lockfile)
 
 	server_add_accept(0);
 
-	set_signals(server_signal_callback);
-	server_loop();
+	proc_loop(server_proc, server_loop);
 	status_prompt_save_history();
 	exit(0);
 }
 
-/* Main server loop. */
-void
-server_loop(void)
-{
-	while (!server_should_shutdown()) {
-		log_debug("event dispatch enter");
-		event_loop(EVLOOP_ONCE);
-		log_debug("event dispatch exit");
-
-		server_client_loop();
-	}
-}
-
-/* Check if the server should exit (no more clients or sessions). */
+/* Server loop callback. */
 int
-server_should_shutdown(void)
+server_loop(void)
 {
 	struct client	*c;
 
-	if (!options_get_number(&global_options, "exit-unattached")) {
+	server_client_loop();
+
+	if (!options_get_number(global_options, "exit-unattached")) {
 		if (!RB_EMPTY(&sessions))
 			return (0);
 	}
@@ -269,9 +217,9 @@ server_should_shutdown(void)
 	return (1);
 }
 
-/* Shutdown the server by killing all clients and windows. */
+/* Exit the server by killing all clients and windows. */
 void
-server_send_shutdown(void)
+server_send_exit(void)
 {
 	struct client	*c, *c1;
 	struct session	*s, *s1;
@@ -279,10 +227,10 @@ server_send_shutdown(void)
 	cmd_wait_for_flush();
 
 	TAILQ_FOREACH_SAFE(c, &clients, entry, c1) {
-		if (c->flags & (CLIENT_BAD|CLIENT_SUSPENDED))
+		if (c->flags & CLIENT_SUSPENDED)
 			server_client_lost(c);
 		else
-			server_write_client(c, MSG_SHUTDOWN, NULL, 0);
+			proc_send(c->peer, MSG_SHUTDOWN, -1, NULL, 0);
 		c->session = NULL;
 	}
 
@@ -328,7 +276,7 @@ server_update_socket(void)
 
 /* Callback for server socket. */
 void
-server_accept_callback(int fd, short events, unused void *data)
+server_accept(int fd, short events, __unused void *data)
 {
 	struct sockaddr_storage	sa;
 	socklen_t		slen = sizeof sa;
@@ -349,7 +297,7 @@ server_accept_callback(int fd, short events, unused void *data)
 		}
 		fatal("accept failed");
 	}
-	if (server_shutdown) {
+	if (server_exit) {
 		close(newfd);
 		return;
 	}
@@ -369,26 +317,26 @@ server_add_accept(int timeout)
 		event_del(&server_ev_accept);
 
 	if (timeout == 0) {
-		event_set(&server_ev_accept,
-		    server_fd, EV_READ, server_accept_callback, NULL);
+		event_set(&server_ev_accept, server_fd, EV_READ, server_accept,
+		    NULL);
 		event_add(&server_ev_accept, NULL);
 	} else {
-		event_set(&server_ev_accept,
-		    server_fd, EV_TIMEOUT, server_accept_callback, NULL);
+		event_set(&server_ev_accept, server_fd, EV_TIMEOUT,
+		    server_accept, NULL);
 		event_add(&server_ev_accept, &tv);
 	}
 }
 
 /* Signal handler. */
 void
-server_signal_callback(int sig, unused short events, unused void *data)
+server_signal(int sig)
 {
 	int	fd;
 
 	switch (sig) {
 	case SIGTERM:
-		server_shutdown = 1;
-		server_send_shutdown();
+		server_exit = 1;
+		server_send_exit();
 		break;
 	case SIGCHLD:
 		server_child_signal();
@@ -441,7 +389,7 @@ server_child_exited(pid_t pid, int status)
 		TAILQ_FOREACH(wp, &w->panes, entry) {
 			if (wp->pid == pid) {
 				wp->status = status;
-				server_destroy_pane(wp);
+				server_destroy_pane(wp, 1);
 				break;
 			}
 		}
